@@ -3,8 +3,9 @@
  * Copyright (c) 2009 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation. See COPYING file for details.
+ * modify it under the terms of the GNU General Public License version 2 
+ * or later as published by the Free Software Foundation. 
+ * See COPYING file for details.
  */
 
 #include "libisofs.h"
@@ -73,6 +74,8 @@ int iso_image_new(const char *name, IsoImage **image)
         img->volset_id = strdup(name);
         img->volume_id = strdup(name);
     }
+    img->system_area_data = NULL;
+    img->system_area_options = 0;
     img->builder_ignore_acl = 1;
     img->builder_ignore_ea = 1;
     img->inode_counter = 0;
@@ -330,6 +333,16 @@ int iso_image_get_msg_id(IsoImage *image)
     return image->id;
 }
 
+int iso_image_get_system_area(IsoImage *img, char system_area_data[32768],
+                              int *options, int flag)
+{
+    *options = img->system_area_options;
+    if (img->system_area_data == NULL)
+        return 0;
+    memcpy(system_area_data, img->system_area_data, 32768);
+    return 1;
+}
+
 static
 int dir_update_size(IsoImage *image, IsoDir *dir)
 {
