@@ -2,8 +2,9 @@
  * Copyright (c) 2007 Vreixo Formoso
  *
  * This file is part of the libisofs project; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation. See COPYING file for details.
+ * modify it under the terms of the GNU General Public License version 2 
+ * or later as published by the Free Software Foundation. 
+ * See COPYING file for details.
  */
 #include <stdlib.h>
 #include <sys/types.h>
@@ -152,6 +153,8 @@ const char *iso_error_to_msg(int errcode)
         return "Try to set the boot image of an already bootable image";
     case ISO_BOOT_IMAGE_NOT_VALID:
         return "Trying to use an invalid file as boot image";
+    case ISO_BOOT_IMAGE_OVERFLOW:
+        return "Too many boot images added";
     case ISO_FILE_ERROR:
         return "Error on file operation";
     case ISO_FILE_ALREADY_OPENED:
